@@ -1,5 +1,5 @@
 from datetime import datetime, timedelta
-from typing import Any, List, Tuple, Mapping, Optional
+from typing import Any, List, Tuple, Mapping, Optional, Iterable
 from typing import Union
 
 from .keywords import _KUSTO_KEYWORDS
@@ -16,7 +16,8 @@ MappingType = Union[Mapping, '_MappingExpression']
 DatetimeType = Union[datetime, '_DatetimeExpression']
 TimespanType = Union[timedelta, '_TimespanExpression']
 DynamicType = Union[ArrayType, MappingType]
-OrderedType = Union[DatetimeType, TimespanType, NumberType, StringType]
+ComparableType = Union[DatetimeType, TimespanType, NumberType]
+OrderedType = Union[ComparableType, StringType]
 
 
 # All classes in the same file to prevent circular dependencies
@@ -176,7 +177,7 @@ class BaseExpression:
         """
         return _NumberExpression(KQL(f'tolong({self.kql})'))
 
-    def assign_to_single_column(self, column: '_AnyTypeColumn') -> '_AssignmentToSingleColumn':
+    def assign_to_single_column(self, column: 'BaseColumn') -> '_AssignmentToSingleColumn':
         return _AssignmentToSingleColumn(column, self)
 
     def assign_to_multiple_columns(self, *columns: '_AnyTypeColumn') -> '_AssignmentBase':
@@ -185,7 +186,7 @@ class BaseExpression:
         """
         raise ValueError("Only arrays can be assigned to multiple columns")
 
-    def assign_to(self, *columns: '_AnyTypeColumn') -> '_AssignmentBase':
+    def assign_to(self, *columns: 'BaseColumn') -> '_AssignmentBase':
         if len(columns) == 0:
             # Unspecified column name
             return _AssignmentBase(None, self)
@@ -690,6 +691,12 @@ class _DatetimeExpression(BaseExpression):
         """
         return _DatetimeExpression(KQL(f'startofyear({self.kql})' if offset is None else 'startofyear({self.kql}, {to_kql(offset)})'))
 
+    def day_of_week(self) -> '_TimespanExpression':
+        """
+        https://docs.microsoft.com/en-us/azure/data-explorer/kusto/query/dayofweekfunction
+        """
+        return _TimespanExpression(KQL(f'dayofweek({self.kql})'))
+
 
 @_plain_expression(_KustoType.TIMESPAN)
 class _TimespanExpression(BaseExpression):
@@ -714,6 +721,18 @@ class _TimespanExpression(BaseExpression):
         # https://docs.microsoft.com/en-us/azure/data-explorer/kusto/query/datetime-timespan-arithmetic
         return _TimespanExpression.binary_op(other, ' - ', self)
 
+    def __lt__(self, other: TimespanType) -> _BooleanExpression:
+        return _BooleanExpression.binary_op(self, ' < ', other)
+
+    def __le__(self, other: TimespanType) -> _BooleanExpression:
+        return _BooleanExpression.binary_op(self, ' <= ', other)
+
+    def __gt__(self, other: TimespanType) -> _BooleanExpression:
+        return _BooleanExpression.binary_op(self, ' > ', other)
+
+    def __ge__(self, other: TimespanType) -> _BooleanExpression:
+        return _BooleanExpression.binary_op(self, ' >= ', other)
+
     def ago(self) -> _DatetimeExpression:
         """
         https://docs.microsoft.com/en-us/azure/data-explorer/kusto/query/agofunction
@@ -794,7 +813,7 @@ class _ArrayExpression(_BaseDynamicExpression):
         """
         return self.contains(other)
 
-    def assign_to_multiple_columns(self, *columns: '_AnyTypeColumn') -> '_AssignmentToMultipleColumns':
+    def assign_to_multiple_columns(self, *columns: 'BaseColumn') -> '_AssignmentToMultipleColumns':
         return _AssignmentToMultipleColumns(columns, self)
 
 
@@ -823,14 +842,25 @@ class _MappingExpression(_BaseDynamicExpression):
 
 class _DynamicExpression(_ArrayExpression, _MappingExpression):
     def __getitem__(self, index: Union[StringType, NumberType]) -> 'AnyExpression':
-        return super().__getitem__(index)
+        return _BaseDynamicExpression.__getitem__(self, index)
 
 
-class AnyExpression(
-    _NumberExpression, _BooleanExpression,
-    _StringExpression, _DynamicExpression,
-    _DatetimeExpression, _TimespanExpression
-):
+class _ComparableExpression(_NumberExpression, _DatetimeExpression, _TimespanExpression):
+    # TODO: Implement for strings using 'strcmp': https://docs.microsoft.com/en-us/azure/data-explorer/kusto/query/strcmpfunction
+    def __lt__(self, other: ComparableType) -> _BooleanExpression:
+        return _BooleanExpression.binary_op(self, ' < ', other)
+
+    def __le__(self, other: ComparableType) -> _BooleanExpression:
+        return _BooleanExpression.binary_op(self, ' <= ', other)
+
+    def __gt__(self, other: ComparableType) -> _BooleanExpression:
+        return _BooleanExpression.binary_op(self, ' > ', other)
+
+    def __ge__(self, other: ComparableType) -> _BooleanExpression:
+        return _BooleanExpression.binary_op(self, ' >= ', other)
+
+
+class AnyExpression(_BooleanExpression, _ComparableExpression, _StringExpression, _DynamicExpression):
     pass
 
 
@@ -909,22 +939,22 @@ class _AssignmentBase:
 
 
 class _AssignmentToSingleColumn(_AssignmentBase):
-    def __init__(self, column: '_AnyTypeColumn', expression: ExpressionType) -> None:
+    def __init__(self, column: 'BaseColumn', expression: ExpressionType) -> None:
         super().__init__(column.kql, expression)
 
 
 class _AssignmentFromColumnToColumn(_AssignmentToSingleColumn):
-    def __init__(self, target: '_AnyTypeColumn', source: 'BaseColumn') -> None:
+    def __init__(self, target: 'BaseColumn', source: 'BaseColumn') -> None:
         super().__init__(target, source)
 
 
 class _AssignmentToMultipleColumns(_AssignmentBase):
-    def __init__(self, columns: Union[List['_AnyTypeColumn'], Tuple['_AnyTypeColumn']], expression: ArrayType) -> None:
+    def __init__(self, columns: Iterable['BaseColumn'], expression: ArrayType) -> None:
         super().__init__(KQL(f'({", ".join(c.kql for c in columns)})'), expression)
 
 
 class _AssignmentFromAggregationToColumn(_AssignmentBase):
-    def __init__(self, column: Optional['_AnyTypeColumn'], aggregation: AggregationExpression) -> None:
+    def __init__(self, column: Optional['BaseColumn'], aggregation: AggregationExpression) -> None:
         super().__init__(None if column is None else column.kql, aggregation)
 
 
@@ -949,7 +979,7 @@ class BaseColumn(BaseExpression):
     def as_subexpression(self) -> KQL:
         return self.kql
 
-    def assign_to_single_column(self, column: '_AnyTypeColumn') -> '_AssignmentFromColumnToColumn':
+    def assign_to_single_column(self, column: 'BaseColumn') -> '_AssignmentFromColumnToColumn':
         return _AssignmentFromColumnToColumn(column, self)
 
     def __repr__(self) -> str:
@@ -998,7 +1028,7 @@ class _TimespanColumn(BaseColumn, _TimespanExpression):
     pass
 
 
-class _SubtractableColumn(_NumberColumn, _DatetimeColumn, _TimespanColumn):
+class _SubtractableColumn(_NumberColumn, _DatetimeColumn, _TimespanColumn, _ComparableExpression):
     @staticmethod
     def __resolve_type(type_to_resolve: Union['NumberType', 'DatetimeType', 'TimespanType']) -> Optional[_KustoType]:
         # noinspection PyTypeChecker
